--[[
Main.lua
#################################################################################
What this file does:                                                            #
-This is the file that contains all files. Literally!                           #
-This file will:                                                                #
-Load all other files for all functions of the map, including data files.       #
-Load all graphics and store them in arrays.                                    #
-Handles all math, keyboard functions, graphic drawing and constantly           #
updates graphics and math functions.                                            #
-Handles errors and corrects them or crashes the client.                        #
-Manages loading screen and saves.                                              #
                                                                                #
Save storing, account validation, and updates are handled by the launcher, NOT  #
THE GAME.                                                                       #
                                                                                #
What is GameState?:                                                             #
One of the key concepts in how we handle the game loop, and flow of the game is #
an array title GAMESTATE[]. GAMESTATE consists of 3 values. The values are:     #
Phase - Refers to whether or not you are on a menu, the map, or in a battle.    #
	- This value is either 0 for map, 1 for menu, or 2 for battle. Any other    #
		state will crash the game. Be VERY careful.                             #
Name - Each phase is individually named. These names are useful in breaking the #
		phases down even further by name of the phase. Each phase has multiple  #
		names, and they refer to specific scenarios.                            #
Step - All names will start on step 0, and if necessary go up from there. This  #
		lets us control where we are in the update function. Sometimes this     #
		part won't even matter, however in some large operations it can help us #
		divide the code and keep it clean.                                      #
		                                                                        #
Why this structure?         			          								#
A very good reasoning behind this type of gamestate structure is that we can    #
just jump into editing the code. This type of structure will deliminate what    #
every section does simply by reading the code values. It gives us a solid       #
'if nesting' structure in which to build the code bits in. However, it's very   #
important to note that the functions that use GAMESTATE are NOT directly        #
related in the sense of the game. The functions manage the structure of loading #
files, and executing code, not editing values. This is important, as this type  #
of organization is simply for the sake of keeping our "under the hood" code in  #
a simplistic, readable state so that it can be tweaked, added to or cut down if #
need be. It is very easy to confuse GAMESTATE with a direct relation to the     #
game's functioning code as is operating. DO NOT DO IT. YOU WILL BREAK THE CODE! #
																				#
																				#
Explanation of functions specifically in "Main.lua":                            #
This sections explains, in great detail, what each function does. The reason    #
that this is done is because updating this game is a very difficult task if we  #
don't document things in this manner. That being said, all functions will be    #
commented as to what they do. It is only in the "Main.lua" file that we do      #
documentation on the functions, aside from structure functions. Those can be    # 
found in the Data sub-directory of the scripts folder. These functions are huge #
and require this type of documentation because it is very easy to get confused  #
as to what they do. This is due to the fact these functions handle every single #
possible evaluation and use a specific interface for doing this. This is        #
explained above in the section titled: "What is Gamestate?".                    #
                                                                                #
The following sections are broken into functions, starting with love.load(),    #
and finishing with love.update().                                               #
																				#
function love.load()															#
																				#
What it does:																	#
	love.load is broken into sections, just for organization.					#
																				#
	The first section loads the data. This is a hefty overhead generator. The   #
	data libraries are seemingly large arrays with key indexes that we use with #
	data interpreters to search for indexes. These data libraries are the       #
	details in the game, the item names, the stat bonuses, the hero configs,    #
	etc. The data libraries must be called with require.                        #
																				#
	The second section loads data interpreters. These functions look at data    #
	and tell the game what needs to be done, typically by setting flags for the #
	next update cycle. These are typically not alone, so this section is small. #
	If you create a data interpreter, odds are you will put it with the code    #
	that is interprets for. The exception is general interpreters, and those    #
	are seemingly general data readers. They are very rare in the source code.  #
																				#
	The third section loads the functional code used in the game loop, run off  #
	of flags and values. This code is executed under fine conditions.           #
																				#
	The fourth section sets up our images. This is VERY IMPORTANT. They MUST BE #
	LOADED TO THE PROPER TABLE, USING THE NAME AS THE KEY! This standard is so  #
	very important for graphic display and they MUST follow this practice. An   #
	example would be: 															#
		UI[loadingscreen] = love.graphics.newImage("loadingscreen.png")			#
		with the CORRECT pathing being ../Resources/Graphics/System/UI			#
	This section will produce more overhead than the third section.             #
																				#
	The fifth section loads audio. This is similar to the fourth section, but   #
	the code is: audio = love.audio.newSource("audio.mp3", type).               #
	NOTE: FOR TYPE, STATIC FOR SMALL SOUNDS, STREAM FOR LARGE.                  #
																				#
	The sixth section initializes flags to load the start screen, and plays the #
	start screen music.															#
																				#
function love.draw()															#
																				#
What it does:																	#
	love.draw is broken similarly down into sections. The sections however are  #
	The interface for the breakdown is by type of image. At the top, we do map  #
	drawing. This is because the map is the very bottom level of the graphics.  #
																				#
	The second section renders the details of the map. These are extra graphics #
	on top of the first level of the graphics. They add effects to the map such #
	as flowers, cracks, book cases, banners, or flags.                          #
																				#
	The third section renders interactable graphics. These are key values in an #
	array that holds a specific instance of the object in different tables. The #
	looping for this is very ugly, but it is the only way that we can ensure    #
	the object we are interacting with is in the correct coordinates. Otherwise #
	all objects of the type would be called.									#
																				#
	The fourth section renders people grahpics. This includes the player amd    #
	the NPC's in towns, and other sections of the game. These are done like the #
	interactables are, and the loops can be messy. However, there are very few  #
	duplications of these graphics. This is because monsters are random results #
	from an arbitrary algorithm involving steps and various other variables.    #
																				#
	The fifth section renders AI and screens. This is because these are shown   #
	above everything else, and must be rendered last. If they were rendered     #
	first, the map, characters, and interactables would cover it entirely.      #
																				#
function love.update()															#
																				#
	This is the meat of the program. This function handles all other functions  #
	and ensures that loops are working properly. The update functions in three  #
	parts, depending on which game state we are currently in. The function runs #
	different functions based on what part we are in, and is super dependent on #
	flags. Editing the love.update function is the most tedious thing to do in  #
	the updating process because of the complexity of the mathematical          #
	implications. The flags must be exact, and the function calls must always   #
	use "dt" to change values, otherwise the game will catastrophically crash.  #
	Updating this function in future patches and while adding more content must #
	be very well thought out, as it can destroy something unintentionally. This #
	function is very important. The game states will break down even further,   #
	and the nesting of ifs can be confusing. ALL parts of the update function   #
	must be commented so that it can be easily tracked. The level of commenting #
	required is incredibly high, and you must do it for every section. If it is #
	not done, it will become very difficult to edit the function later.			#
																				#
	Understanding the Music play method.                                        #
	In love.update(), 2 flags indicate music, MUSICNAME, and ISPLAYING. We see  #
	if ISPLAYING matches MUSICNAME. If not, we stop the music, and change it.   #
	This system ensures that we don't repeat play songs in love.update short    #
	iterations. It also adds safety to playing music, and manages streams well. #
	However, note that when we are in battle phase, sounds can be play over the #
	music, so we don't use these flags. we normally will call the function in a #
	native way, so that where the sound should play (i.e the attack function),  #
	the sound will be played from that function, as we don't have the           #
	constraints that we do with graphics.                                       #	
]]

function love.load()
    require "test"
    require "Scripts.Development.Debugger"
    require "Scripts.Structure.Unit"
	require "Scripts.Stat Management.Stats"
	require "Scripts.Quests.Lookup"
	DEBUG_MODE = true
	DEBUG_LEVEL = 3
	DEBUG_UNIT = tUnit.new{level=1, HP=1, MaxHP=1, MP=1, MaxMP=1, strength=1, agility=1, spirit=1, armor=1, armorPen=1, name="Debugger", graphic=1, face=1, voice=1, x=0, y=0, map=0, bMoves=true}
end

function love.keypressed(key, unicode)
	if key == 'up' or key == 'down' or key == 'left' or key == 'right' then
		DEBUG_UNIT.Move(key, 0, nil, nil, nil)
	end
end

function love.draw()
    local iterator
	love.graphics.print("Test", 200, 200)
	  if DEBUG_PUSHPRINT then
	      DEBUG_PUSHPRINT = false
	      love.graphics.clear()
	      iterator = 0
	      while iterator <= DEBUG_MAX do
	          love.graphics.print(DEBUG_LINE[iterator], 100*iterator, 100)
	          iterator = iterator + 1
	      end
	  end 
end

function love.update(dt)
end     